""" zgMap - Google Maps wrapper """

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Globals import DTMLFile
from Acquisition import Implicit, aq_base
from Globals import Persistent
from OFS.SimpleItem import SimpleItem
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from App.Common import package_home
from AccessControl.Role import RoleManager
from Products.PythonScripts.standard import url_quote
from Products.PythonScripts.standard import url_unquote
from Products.PythonScripts.PythonScript import manage_addPythonScript
import urllib

def addzgMap(self, id, title, initLat, initLon, initZoom, mapsKey, REQUEST=None):
    """ Add Google Map Instance """
    ob = zgMap(id, initLat, initLon, initZoom, mapsKey)
    ob.title = str(title)
    id = self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')
        
manage_addzgMap = DTMLFile('dtml/zgMapAdd', globals())

class zgMap(Folder):
	""" Google Maps container """

	meta_type = 'zgMap'
	security=ClassSecurityInfo()
		
	manage_options = [
		{'action': 'manage_main', 'help': ('OFSP', 'ObjectManager_Contents.stx'), 'label': 'Contents'},
		{'action': 'manage_displayMap', 'label': 'Preview Map'},
		{'action': 'manage_propertiesForm', 'help': ('OFSP', 'Properties.stx'), 'label': 'Properties'},
		{'action': 'manage_access', 'help': ('OFSP', 'Security.stx'), 'label': 'Security'},
		{'action': 'manage_owner', 'help': ('OFSP', 'Ownership.stx'), 'label': 'Ownership'},
		{'action': 'manage_displayIcons', 'label': 'Icons'},
		{'action': 'manage_displayObjects', 'label': 'Objects'},
		{'action': 'manage_geoCode', 'label': 'GeoCode'},
		{'action': 'manage_googleCode', 'label': 'GoogleCode'},
		]

		
	_properties = (
        {'id':'title', 'type': 'string', 'mode': 'w'},
        {'id':'initLat', 'type': 'string', 'mode': 'w'},
        {'id':'initLon', 'type': 'string', 'mode': 'w'},
        {'id':'initZoom', 'type': 'string', 'mode': 'w'},
        {'id':'mapsKey', 'type': 'string', 'mode': 'w'},
        )

	manage_displayIcons = DTMLFile('dtml/displayIcons', globals())
	manage_displayObjects = DTMLFile('dtml/displayObjects',globals())
	manage_displayMap = DTMLFile('dtml/displayMap',globals())
	manage_geoCode = DTMLFile('dtml/geoCode', globals())
	manage_googleCode = DTMLFile('dtml/googleCode', globals())


	def __init__(self, id, initLat, initLon, initZoom, mapsKey):
		self.id = id
		self.initLat = initLat
		self.initLon = initLon
		self.initZoom = initZoom
		self.mIcons = []
		self.mapsKey = mapsKey
		
		filecontents = open('Products/zgMap/files/addPoint.py', 'r').read()
		manage_addPythonScript(self, 'addPoint')
		self._getOb('addPoint').write(filecontents)
		
		filecontents = open('Products/zgMap/files/getPoints.py', 'r').read()
		manage_addPythonScript(self, 'getPoints')
		self._getOb('getPoints').write(filecontents)

	security.declarePublic('addToMap')
	def addToMap(self, point):
		"""  Returns a string to add the passed point to a map """
		return """
				map.addOverlay(createMarker(new GPoint(""" + str(point["lon"]) + """, """ + str(point["lat"]) + """), \"""" + str(point["html"]).replace("""\n"""," ") + """\", gm""" + str(point["type"]) + """));
		"""
		
	security.declarePublic('postcodeAnywhere')
	def postcodeAnywhere(self, account_code, license_code, action, postcode, building, accuracy):
		"""Use Postcode anywhere"""
		postcode = "".join(postcode.split(" "))
		info = urllib.urlopen("http://services.postcodeanywhere.co.uk/csv.aspx?account_code=" + account_code + "&license_code=" + license_code + "&action=" + action + "&postcode=" + postcode + "&building=" + building + "&accuracy=" + accuracy).read()
		info = "".join(info.split("\r"))
		info = "".join(info.split("\""))
		info = info.split("\n")
		info[0] = info[0].split(",")
		info[1] = info[1].split(",")
		resp = {}
		for a in range(0,len(info[0])):
		  resp[info[0][a]] = info[1][a]
		return resp

	def getll(self, postcode):
		""" Grab the coordinates of a postcode """
		try:
		  postcode = "+".join(postcode.split(" "))
		  info = urllib.urlopen("http://maps.google.co.uk/maps?q=" + postcode + "&output=js").read()
		  info = info.split("lat: ")[3]
		  info = info.split(",lng: ")
		  lat = info[0]
		  lon = info[1].split(",")[0]
		  return (lat,lon)
		except:
		  raise "Could not resolve postcode"

	security.declareProtected('zgMap: Create Icon', 'newIcon')
	def newIcon(self, name, image, shadow, size, shadowsize, anchor, windowanchor):
		""" Adds an icon definition """
		try:
		  iconnames = [a['name'] for a in self.mIcons]
		  if name in iconnames:
		    b = "Icon Exists"
		    return
		except:
		  pass
		b = """
		var """ + name + """ = new GIcon();
		""" + name + """.image = \"""" + str(image) + """\";
		""" + name + """.shadow = \"""" + str(shadow) + """\";
		""" + name + """.iconSize = new GSize""" + str(size) + """;
		""" + name + """.shadowSize = new GSize""" + str(shadowsize) + """;
		""" + name + """.iconAnchor = new GPoint""" + str(anchor) + """;
		""" + name + """.infoWindowAnchor = new GPoint""" + str(windowanchor) + """;
		"""
		addIcon = {"name":name,"image":image,"shadow":shadow,"iconSize":str(size),"shadowSize":str(shadowsize),"iconAnchor":str(anchor),"infoWindowAnchor":str(windowanchor)}
		sofar = []
		try:
		  for a in self.mIcons:
		    sofar.append(a)
		except:
		  pass
		sofar.append(addIcon)
		self.mIcons = sofar
		return b

	security.declareProtected('zgMap: Delete Icon', 'deleteIcon')
	def deleteIcon(self, name):
		""" Remove an icon definition """
		try:
		  self.mIcons = [a for a in self.mIcons if a['name'] != name]
		except:
		  pass
		return "Deleted"

	security.declarePublic('declareIcons')
	def declareIcons(self):
		""" Outputs all the icon declarations """
		c = """
			function resolveIconName(icn) 
			{
				switch(icn)
				{
"""
		b = """ 
		"""
		for a in self.mIcons:
		  name = "gm" + a['name']
		  image = a['image']
		  shadow = a['shadow']
		  size = a['iconSize']
		  shadowsize = a['shadowSize']
		  anchor = a['iconAnchor']
		  windowanchor = a['infoWindowAnchor']
		  c = c + """
					case \"""" + name + """\":
						return """ + name + """
						break
"""
		  b = b + """
			var """ + name + """ = new GIcon();
			""" + name + """.image = \"""" + str(image) + """\";
			""" + name + """.shadow = \"""" + str(shadow) + """\";
			""" + name + """.iconSize = new GSize""" + str(size) + """;
			""" + name + """.shadowSize = new GSize""" + str(shadowsize) + """;
			""" + name + """.iconAnchor = new GLatLng""" + str(anchor) + """;
			""" + name + """.infoWindowAnchor = new GLatLng""" + str(windowanchor) + """;
			"""
		b = b + """

		"""
		c = c + """
				}
			}
"""
		b = b + c
		return b
		
		
	security.declarePublic('populateFromXml')
	def populateFromXml(self):
		""" Grabs the XML output file, and parses it """
		outstr = """
		function loadXMLpoints() {
			var request = GXmlHttp.create();
			document.getElementById("info").innerHTML = '<div id="overmap"><strong>Loading...</strong></div>';
			request.open("GET", "./""" + self.id + """/XMLout", true);
			request.onreadystatechange = function() 
			{
				if (request.readyState == 4) 
				{
					if (request.status==200)
					{
						var ok = 0;
						var xmlDoc = request.responseXML;
						var maptag  = xmlDoc.documentElement.getElementsByTagName("marker");
						map.setCenter(new GLatLng(maptag[0].getAttribute("lat") ,maptag[0].getAttribute("lon")), 13);
						var markers = xmlDoc.documentElement.getElementsByTagName("marker");
						for (var i = 0; i < markers.length; i++)
						{
							ok = 1;
							var mgx = GXml.value(markers[i]);
							var point = new GPoint(parseFloat(markers[i].getAttribute("lat")),  parseFloat(markers[i].getAttribute("lon")));
							var marker = createMarker(point, unescape(markers[i].getAttribute("html")), resolveIconName(markers[i].getAttribute("type")));
							map.addOverlay(marker);
							mhtml = unescape(markers[i].getAttribute("html"));
							if (markers[i].getAttribute("show") == "true")
							{
								marker.openInfoWindowHtml(mhtml);
							}
							document.getElementById("info").innerHTML = mgx;
						}
						if (ok == 0)
						{
							document.getElementById("info").innerHTML = '<div id="overmap" style="background-color: red; color: white;"><strong>Failed</strong></div>';
						}
					}
					else
					{
						document.getElementById("info").innerHTML = '<div id="overmap" style="background-color: red; color: white;"><strong>Failed</strong></div>';
					}
				}
			}
			request.send(null);
		}
		"""
		return outstr
		
	security.declarePublic('XMLout')
	def XMLout(self):
		""" Writes out all the points in an XML format """
		self.REQUEST.RESPONSE.setHeader("Content-type", "text/xml")
		self.REQUEST.RESPONSE.setHeader("Expires", "-1xm")
		self.REQUEST.RESPONSE.setHeader("Cache-control", "no-store")
		self.REQUEST.RESPONSE.write("""<?xml version="1.0" encoding="UTF-8"?>""")
		intro = ""
		mypoints = self.getPoints()
		try:
			intro = "<map lat=\"%s\" lon=\"%s\">" % (mypoints["cen"]["lat"], mypoints["cen"]["lon"])
		except:
			intro = "<map lat=\"%s\" lon=\"%s\">" % (self.initLat, self.initLon)
		self.REQUEST.RESPONSE.write(intro)
		for point in mypoints["pnts"]:
			try:
			  if point["show"] == "true":
			    pass
			  else:
			    raise "1"
			except:
			  point["show"] = "false"
			try:
				point["html"] = url_quote(point["html"])
				self.REQUEST.RESPONSE.write("<marker show=\"" + str(point["show"]) + "\" lat=\"" +str(point["lon"]) + "\" lon=\"" + str(point["lat"]) + "\" type=\"gm" + str(point["type"]) + "\" html=\"" + str(point["html"]) + """\" />
""")
			except:
				pass
		self.REQUEST.RESPONSE.write("</map>")
		return " "
			
	security.declarePublic('writeMap')
	def writeMap(self):
		return """<div id="map" style="width: 500px; height: 400px"></div><div id="info"></div>"""
			
	security.declarePublic('displayMap')
	def displayMap(self):
		""" Writes javascript code to create the map on-screen. """
		myicons = self.declareIcons()
		op = """
			<style type="text/css">
			div#overmap
			{
				position: relative;
				top: -225px;
				left: 200px;
				padding: 10px;
				font-family: Georgia, sans;
				border: 3px solid black;
				background-color: white;
				opacity: 0.75;
				text-align: center;
				-moz-border-radius: 3px;
				width: 100px;
			}
			</style>
			<script src="http://maps.google.com/maps?file=api&v=2&key=%s\" type="text/javascript"></script>
			
			<script type="text/javascript">
			var map = new GMap2(document.getElementById("map"));
		""" % (self.mapsKey)
		self.declareIcons()
		op = op + """
			window.onload = function() {
				map.addControl(new GMapTypeControl());
				map.addControl(new GLargeMapControl());
				map.addControl(new GScaleControl());
				map.enableDoubleClickZoom();
				map.enableContinuousZoom();
				map.setCenter(new GLatLng(""" + str(self.initLat) + "," + str(self.initLon) + """), """+str(17-int(self.initZoom))+""");
				loadXMLpoints();
				
			}
			"""
		op = op + myicons
		op = op + """
			function createMarker(point, phtml, mt) {
			  var marker = new GMarker(point, mt);
			  var html = phtml;
			  if (html != "")
			  {
				  GEvent.addListener(marker, 'click', function() {
						document.getElementById("info").innerHtml = phtml;
					  marker.openInfoWindowHtml(phtml);
					});
			}
			  return marker;
			}
		"""
		op = op + self.populateFromXml()
		op = op + """
			</script>"""
		return op

	security.declarePublic('displayMapGC')
	def displayMapGC(self):
		""" Writes javascript code to create a map for geocoding into #message. """
		op = """
			<style type="text/css">
			div#overmap
			{
				position: relative;
				top: -200px;
				left: 200px;
				padding: 10px;
				border: 3px solid black;
				background-color: white;
				opacity: 0.75;
				-moz-border-radius: 5px;
				width: 100px;
			}
			</style>
			<div id="map" style="width: 500px; height: 400px"></div><div id="info"></div><script src="http://maps.google.com/maps?file=api&v=2&key=%s" type="text/javascript"></script>"""  % (self.mapsKey) 
		op = op + """
			<script type="text/javascript">
			var map = new GMap(document.getElementById("map"));
			GEvent.addListener(map, "moveend", function() {
			var center = map.getCenterLatLng();
			map.addControl(new GMapTypeControl());
			var latLngStr = '<dl><dt>Latitude</dt><dd>' + center.y + '</dd><dt>Longitude</dt><dd>' + center.x + '</dd></dl><input type="hidden" name="lat" value="' + center.y + '" /><input type="hidden" name="lon" value="' + center.x + '" />' 
			document.getElementById("message").innerHTML = latLngStr;
			map.clearOverlays();
			map.addOverlay(new GMarker(center));
			});
			map.addControl(new GSmallMapControl());
			map.setCenter(new GLatLng(""" + str(self.initLat) + "," + str(self.initLon) + """), """+str(17-int(self.initZoom))+""");
			</script>"""
		return op

	security.declarePublic('createMap') 
	def createMap(self):
		""" Changes the object with id=id to a google maps instance of name map"""
		return """var map = new GMap(document.getElementById(\"""" + self.id + """\"));"""

	security.declarePublic('centerAndZoom') 
	def centerAndZoom(self, point, level):
		""" Centres the map on a specific point, and sets zoom level"""
		return """map.setCenter(new GLatLng(""" + point["lon"] + "," + point["lat"] + """), """ + str(17-int(level)) + """);"""

	security.declarePublic('getIcons') 
	def getIcons(self):
		""" Return all the icons that have been registered. """
		return self.mIcons

	security.declarePublic('addPointPC')
	def addPointPC(self, id, postcode, type, html):
		""" Calls addPoint with a resolved postcode """
		ll = self.getll(postcode)
		lat = ll[0]
		lon = ll[1]
		return self.addPoint(lat, lon, id, type, html)

	security.declarePublic('getObjects') 
	def getObjects(self):
		""" Returns all known objects """
		return [(a["lat"], a["lon"], a["type"], a["html"].replace("&","&amp;").replace("<","&lt;").replace(">","&gt;")) for a in self.getPoints()["pnts"]]

InitializeClass(zgMap)